package xyz.zhouxy.plusone.jdbc;

import java.io.Serializable;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.ResultSetExtractor;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.jdbc.core.namedparam.SqlParameterSource;

import xyz.zhouxy.plusone.domain.Entity;

public abstract class JdbcEntityDaoSupport<T extends Entity<ID>, ID extends Serializable>
        extends PlusoneJdbcDaoSupport {

    protected RowMapper<T> rowMapper;
    protected ResultSetExtractor<Optional<T>> resultSetExtractor;

    protected JdbcEntityDaoSupport(@Nonnull NamedParameterJdbcTemplate namedParameterJdbcTemplate) {
        super(namedParameterJdbcTemplate);
        this.rowMapper = (ResultSet rs, int rowNum) -> mapRow(rs);
        this.resultSetExtractor = (ResultSet rs) -> rs.next() ? Optional.of(mapRow(rs)) : Optional.empty();
    }

    protected final Optional<T> queryForObject(String sql) {
        return queryForObject(sql, this.resultSetExtractor);
    }

    protected final Optional<T> queryForObject(String sql, SqlParameterSource paramSource) {
        return queryForObject(sql, paramSource, this.resultSetExtractor);
    }

    protected final Optional<T> queryForObject(String sql, String paramName, Object value) {
        return queryForObject(sql, new MapSqlParameterSource(paramName, value), this.resultSetExtractor);
    }

    protected final List<T> queryForList(String sql) {
        return queryForList(sql, this.rowMapper);
    }

    protected final List<T> queryForList(String sql, SqlParameterSource parameterSource) {
        return queryForList(sql, parameterSource, this.rowMapper);
    }

    protected final List<T> queryForList(String sql, String paramName, Object value) {
        return queryForList(sql, new MapSqlParameterSource(paramName, value), this.rowMapper);
    }

    protected final Stream<T> queryForStream(String sql, SqlParameterSource parameterSource) {
        return queryForStream(sql, parameterSource, this.rowMapper);
    }

    protected final Stream<T> queryForStream(String sql, String paramName, Object value) {
        return queryForStream(sql, new MapSqlParameterSource(paramName, value), this.rowMapper);
    }

    protected abstract T mapRow(ResultSet rs) throws SQLException;

    protected void setRowMapper(@Nonnull RowMapper<T> rowMapper) {
        this.rowMapper = rowMapper;
    }

    protected void setResultSetExtractor(@Nonnull ResultSetExtractor<Optional<T>> resultSetExtractor) {
        this.resultSetExtractor = resultSetExtractor;
    }
}
