package xyz.zhouxy.plusone.system.util;

import java.nio.charset.StandardCharsets;
import java.util.Arrays;

import org.springframework.util.Assert;

import com.google.common.hash.Hashing;

import xyz.zhouxy.plusone.util.RandomUtil;

/**
 * 密码工具类
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public final class PasswordUtil {
    private static final char[] SALT_BASE_CHAR_ARRAY = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ~`!@#$%^&*()_-+={}[]|\\:;\"',.<>?/"
            .toCharArray();

    /**
     * 将密码和随机盐混合，并进行哈希加密。
     *
     * @param password 密文密码
     * @param salt     随机盐
     * @return 哈希加密的结果
     */
    public static String hashPassword(String password, String salt) {
        Assert.notNull(password, "Password must not be null");
        Assert.notNull(salt, "Salt must not be null");
        return Hashing.sha512().newHasher()
                .putInt(Arrays.hashCode(salt.toCharArray()))
                .putString(password, StandardCharsets.UTF_8)
                .putInt(password.length())
                .putBoolean(password.length() % 2 == 0)
                .putString(salt, StandardCharsets.UTF_8)
                .putInt(Arrays.hashCode(password.toCharArray()))
                .hash()
                .toString();
    }

    /**
     * 生成 24 位的字符串
     *
     * @return 生成的随机盐
     */
    public static String generateRandomSalt() {
        return RandomUtil.randomStr(SALT_BASE_CHAR_ARRAY, 24);
    }

    private PasswordUtil() {
        // 不允许实例化
        throw new IllegalStateException("Utility class");
    }
}
