package xyz.zhouxy.plusone.system.domain.model.account;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.Objects;
import java.util.Optional;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import lombok.ToString;
import xyz.zhouxy.plusone.domain.IValueObject;

/**
 * 账号详细信息
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
@ToString
public class AccountInfo implements IValueObject {

    @Nullable
    private final Nickname nickname;
    @Nullable
    private final URL avatar;
    @Nonnull
    private final Sex sex;

    private AccountInfo(@Nullable Nickname nickname, @Nullable URL avatar, @Nullable Sex sex) {
        this.nickname = nickname;
        this.avatar = avatar;
        this.sex = Objects.nonNull(sex) ? sex : Sex.UNSET;
    }

    public Optional<Nickname> getNickname() {
        return Optional.ofNullable(nickname);
    }

    public Optional<URL> getAvatar() {
        return Optional.ofNullable(avatar);
    }

    @Nonnull
    public Sex getSex() {
        return this.sex;
    }

    // Builder

    public static Builder builder() {
        return new Builder();
    }

    public static class Builder {
        private Nickname nickname;
        private URL avatar;
        private Sex sex;

        public Builder nickname(@Nullable Nickname nickname) {
            this.nickname = nickname;
            return this;
        }

        public Builder nickname(@Nullable String nickname) {
            return nickname(Nickname.ofNullable(nickname));
        }

        public Builder avatar(@Nullable URL avatar) {
            this.avatar = avatar;
            return this;
        }

        public Builder avatar(@Nullable String avatar) {
            URL avatarURL;
            try {
                avatarURL = Objects.nonNull(avatar) ? new URL(avatar) : null;
            } catch (MalformedURLException e) {
                throw new IllegalArgumentException(e);
            }
            return avatar(avatarURL);
        }

        public Builder sex(@Nullable Sex sex) {
            this.sex = sex;
            return this;
        }

        public AccountInfo build() {
            return new AccountInfo(this.nickname, this.avatar, this.sex);
        }
    }
}
