package xyz.zhouxy.plusone.system.domain.model.account;

import java.util.Objects;
import java.util.regex.Pattern;

import xyz.zhouxy.plusone.commons.annotation.StaticFactoryMethod;
import xyz.zhouxy.plusone.commons.constant.PatternConsts;
import xyz.zhouxy.plusone.domain.ValidatableStringRecord;

/**
 * 值对象：昵称
 *
 * @author <a href="https://gitee.com/zhouxy108">ZhouXY</a>
 */
public class Nickname extends ValidatableStringRecord {

    public static final Pattern REGEX = PatternConsts.NICKNAME;

    private Nickname(String value) {
        super(REGEX);
        if (value == null) {
            throw new IllegalArgumentException("昵称不能为空");
        }
        this.value = value;
        if (!isValid()) {
            throw new IllegalArgumentException("昵称格式错误");
        }
    }

    @StaticFactoryMethod(Nickname.class)
    public static Nickname of(String nickname) {
        return new Nickname(nickname);
    }

    @StaticFactoryMethod(Nickname.class)
    public static Nickname ofNullable(String nickname) {
        return Objects.nonNull(nickname) ? new Nickname(nickname) : null;
    }
}
