package xyz.zhouxy.plusone.system.domain.model.dict;

import java.util.Objects;
import java.util.Set;
import java.util.stream.Collectors;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.util.CollectionUtils;

import xyz.zhouxy.plusone.jdbc.PlusoneJdbcDaoSupport;

class DictValueDAO extends PlusoneJdbcDaoSupport {

    DictValueDAO(@Nonnull NamedParameterJdbcTemplate jdbc) {
        super(jdbc);
    }

    void updateDictValues(Dict entity) {
        update("DELETE FROM sys_dict_value WHERE dict_type = :dictType",
                "dictType", entity.getId().orElseThrow());
        long i = insertDictValues(entity.getId().orElseThrow(), entity);
        assertResultEquals(i, entity.count());
    }

    long insertDictValues(Long dictId, Dict entity) {
        if (Objects.isNull(dictId) || Objects.isNull(entity) || CollectionUtils.isEmpty(entity.getValues())) {
            return 0;
        }
        return batchUpdate(
                "INSERT INTO sys_dict_value (dict_type, dict_key, label) VALUES (:dictType, :dictKey, :label)",
                entity.getValues(),
                dictValue -> new MapSqlParameterSource()
                        .addValue("dictType", dictId)
                        .addValue("dictKey", dictValue.getKey())
                        .addValue("label", dictValue.getLabel()));
    }

    Set<DictValue> selectDictValuesByDictId(long id) {
        return queryForStream(
                "SELECT dict_key, label FROM sys_dict_value WHERE dict_type = :dictType",
                "dictType", id,
                (rs, i) -> DictValue.of(rs.getInt("dict_key"), rs.getString("label")))
                .collect(Collectors.toSet());
    }
}
