package xyz.zhouxy.plusone.system.domain.model.role;

import java.util.Set;
import java.util.stream.Collectors;

import javax.annotation.Nonnull;

import org.springframework.jdbc.core.namedparam.MapSqlParameterSource;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import xyz.zhouxy.plusone.jdbc.PlusoneJdbcDaoSupport;

class RoleMenuRefDAO extends PlusoneJdbcDaoSupport {

    public RoleMenuRefDAO(@Nonnull NamedParameterJdbcTemplate jdbc) {
        super(jdbc);
    }

    Set<MenuRef> selectMenuRefsByRoleId(long roleId) {
        return queryForStream("SELECT menu_id FROM sys_role_menu WHERE role_id = :roleId",
                "roleId", roleId,
                Long.TYPE)
                .map(MenuRef::of)
                .collect(Collectors.toSet());
    }

    void clearRoleMenuRefs(Role entity) {
        update("DELETE FROM sys_role_menu WHERE role_id = :roleId", "roleId", entity.getId().orElseThrow());
    }

    void saveRoleMenuRefs(Long roleId, Role entity) {
        long i = batchUpdate(
                "INSERT INTO sys_role_menu(role_id, menu_id) VALUES (:roleId, :menuId)",
                entity.getMenus(),
                menuRef -> new MapSqlParameterSource()
                        .addValue("roleId", roleId)
                        .addValue("menuId", menuRef.menuId()));
        assertResultEquals(i, entity.getMenus().size());
    }
}
